******************************************************************************
*  No Copyright, released to the Public Domain
*
*  $Workfile$
*
*  Programmer.: The Ghost in the Machine
*  Date.......: 09/17/98
*  Description: Y2KCODE.EXE generated comments for Y2K remediation herein
*
*  *** PVCS revision log section - Do not modify! ***
*  $Log$
*
******************************************************************************
*------------------------------------------------------------------------
Function Main()
*------------------------------------------------------------------------
LOCAL cProgFile, cY2kFile, cLine, cUpper
LOCAL crlf := CHR(13)+CHR(10)
LOCAL fhin, fhout
LOCAL nProb, nLine
LOCAL lComment := .F.
LOCAL cHead

ScrnHeader()
UseDBFs()

FillFileList()

SELECT FILES
GO TOP
DO WHILE .NOT. EOF()
    cProgFile := ALLTRIM(FILES->filename)+"."+ALLTRIM(FILES->ext)
    cY2kFile  := ALLTRIM(FILES->filename)+".Y2K"
    ERASE (cY2kFile)

    ? "  Processing", PADR(cProgFile,13)

    nProb := 0
    nLine := 0

    fhin  := FOPEN(cProgFile,2)
    fhout := FCREATE(cY2kFile)

    // Check for possible initial PVCS Header (add if not found)
    cHead := ""
    DO WHILE "*" = SUBSTR( (cLine := FReadLn(fhin, 1, 254)), 1, 1 )          // [Y2K]
        cHead += cLine
    ENDDO
    FWRITE(fhOut, IIF(("$WORKFILE$" $ UPPER(cHead)), cHead, PVCSHead()+cHead))

    DO WHILE "" <> cLine    // read the next line at end of loop
        cUpper := UPPER(cLine)
        nLine++
        IF (("DATE" $ cUpper) .OR. ("CTOD" $ cUpper) .OR.   ;
            ("DTOC" $ cUpper) .OR. ("DTOS" $ cUpper) .OR.   ;
            ("SUBS" $ cUpper) .OR. ("STUF" $ cUpper) .OR.   ;
            ("@D"   $ cUpper) .OR. ("@ D"  $ cUpper) .OR.   ;
            ("LEFT" $ cUpper) .OR. ("RIGH" $ cUpper) .OR.   ;
            ("TRAN" $ cUpper) .OR. ("STRT" $ cUpper)       );
            .AND. (.NOT. SUBSTR(ALLTRIM(cUpper),1,1) $ "*/")          // [Y2K]

            // A concern here is that we do not want to write comments to
            // split lines.  So we just flag the need, then wait until no
            // more splits have occurred in the command (no more semicolons).
            lComment := .T.
            nProb++
        ENDIF
        cEOL := SUBSTR(ALLTRIM(SUBSTR(cline,1,LEN(cLine)-2)),-1)          // [Y2K]
        IF lComment .AND. cEOL <> ";"
            IF "[Y2K]" $ cLine
                FWRITE(fhout, cLine)
            ELSE
                FWRITE(fhout, SUBSTR(cLine,1,LEN(cLine)-2)+      ;
                              +SPACE(10)+"// [Y2K]"+crlf         )
            ENDIF
            lComment := .F.
        ELSE
            FWRITE(fhout, cLine)
        ENDIF
        cLine := FReadLn(fhin, 1, 254)
    ENDDO

    ?? " -- ", STR(nLine,4,0), "lines"
    ?? " -- ", STR(nProb,4,0), "flagged lines"

    FCLOSE(fhin)
    FCLOSE(fhout)

    SELECT FILES
    SKIP
ENDDO

CleanUp()

RETURN NIL

*------------------------------------------------------------------------
STATIC FUNCTION ScrnHeader()
*------------------------------------------------------------------------
CLS
?" Ŀ"
?"  Producing Y2K Analyses for each PRG.  Commented '*.Y2K' files are created  "
?" "
RETURN nil

*------------------------------------------------------------------------
STATIC FUNCTION UseDBFs()
*------------------------------------------------------------------------
LOCAL aDir := {{"FILENAME", "C", 9, 0}, ;
               {"EXT", "C", 3, 0}}

ERASE ("files.dbf")
DBCREATE("files", aDir)

USE files  NEW EXCLUSIVE

RETURN nil

*------------------------------------------------------------------------
STATIC FUNCTION FillFileList()
*------------------------------------------------------------------------
LOCAL cFiles, i, cLine
LOCAL crlf := CHR(13)+CHR(10)
LOCAL lFile := .F.

RUN ("dir *.prg > files.tmp")
cFiles := MEMOREAD("files.tmp")
ERASE ("files.tmp")
ERASE ("files.txt")
FOR i = 1 TO MLCOUNT(cFiles, 254, 1, .F.)
    cLine := MEMOLINE(cFiles, 254, i, 1, .F.)
    IF 0 <> AT(" PRG  ", cLine)
        MEMOWRIT("files.txt", MEMOREAD("files.txt")+SUBSTR(cLine,1,12)+crlf)          // [Y2K]
        lFile := .T.
    ENDIF
NEXT i

IF .NOT. lFile
    ? "  No PRGs found"
    QUIT
ELSE
    SELECT FILES
    APPEND FROM files.txt SDF
ENDIF
RETURN nil

*------------------------------------------------------------------------
STATIC FUNCTION CleanUp()
*------------------------------------------------------------------------
CLOSE ALL
ERASE ("files.tmp")
ERASE ("files.txt")
ERASE ("files.dbf")
IF 2 = ALERT("Shall we rename the *.Y2K commented files to *.PRG?", {"No", "Yes"})
    IF 0 < ADIR("*.old")
        RUN ("del *.old")
    ENDIF
    RUN ("ren *.prg *.old")
    RUN ("ren *.y2k *.prg")
ENDIF
RETURN nil

*------------------------------------------------------------------------
STATIC FUNCTION PVCSHead()
*------------------------------------------------------------------------
LOCAL crlf := CHR(13)+CHR(10)
LOCAL cHead :=  ;
"******************************************************************************"+crlf+;
"*  Copyright (1998), MetLife Insurance Company                                "+crlf+;
"*  All rights reserved.                                                       "+crlf+;
"*                                                                             "+crlf+;
"*  $Workfile$                                                                 "+crlf+;
"*                                                                             "+crlf+;
"*  Programmer.: The Ghost in the Machine                                      "+crlf+;
"*  Date.......: "+DTOC(DATE())                                                 +crlf+;
"*  Description: Y2KCODE.EXE generated comments for Y2K remediation herein     "+crlf+;
"*                                                                             "+crlf+;
"*  *** PVCS revision log section - Do not modify! ***                         "+crlf+;
"*  $Log$                                                                      "+crlf+;
"*                                                                             "+crlf+;
"******************************************************************************"+crlf          // [Y2K]
RETURN cHead
